package com.friday.tooldriver.mapper.taieoc;

import com.friday.operation.api.keepalive.dto.taieoc.OcTenantFilterDto;
import com.friday.tooldriver.po.taieoc.OcTenantPo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface OcTenantMapper {
    void insert(OcTenantPo po);

    void batchInsert(List<OcTenantPo> poList);
    List<OcTenantPo> selectOcTenant(OcTenantFilterDto filterDto);
    void update(OcTenantPo po);
    void deleteByPrimaryKey(Long id);

    OcTenantPo selectByPrimaryKey(Long id);

    void deleteByOcInfo(@Param("ocInfoId")Long ocInfoId, @Param("operator") String operator);

    int selectTenantCountByOc(Long ocInfoId);
}
